#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import eligibilityresponse
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class EligibilityResponseTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("EligibilityResponse", js["resourceType"])
        return eligibilityresponse.EligibilityResponse(js)
    
    def testEligibilityResponse1(self):
        inst = self.instantiate_from("eligibilityresponse-example-error.json")
        self.assertIsNotNone(inst, "Must have instantiated a EligibilityResponse instance")
        self.implEligibilityResponse1(inst)
        
        js = inst.as_json()
        self.assertEqual("EligibilityResponse", js["resourceType"])
        inst2 = eligibilityresponse.EligibilityResponse(js)
        self.implEligibilityResponse1(inst2)
    
    def implEligibilityResponse1(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-09-16").date)
        self.assertEqual(inst.created.as_json(), "2014-09-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Eligibiliy request could not be processed, please address errors before submitting."))
        self.assertEqual(force_bytes(inst.error[0].code.coding[0].code), force_bytes("a001"))
        self.assertEqual(force_bytes(inst.error[0].code.coding[0].system), force_bytes("http://hl7.org/fhir/adjudication-error"))
        self.assertEqual(force_bytes(inst.form.coding[0].code), force_bytes("ELRSP/2017/01"))
        self.assertEqual(force_bytes(inst.form.coding[0].system), force_bytes("http://national.org/form"))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2503"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/eligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("8812343"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].code), force_bytes("error"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].system), force_bytes("http://hl7.org/fhir/remittance-outcome"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the EligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testEligibilityResponse2(self):
        inst = self.instantiate_from("eligibilityresponse-example-benefits-2.json")
        self.assertIsNotNone(inst, "Must have instantiated a EligibilityResponse instance")
        self.implEligibilityResponse2(inst)
        
        js = inst.as_json()
        self.assertEqual("EligibilityResponse", js["resourceType"])
        inst2 = eligibilityresponse.EligibilityResponse(js)
        self.implEligibilityResponse2(inst2)
    
    def implEligibilityResponse2(self, inst):
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("patient-1"))
        self.assertEqual(force_bytes(inst.contained[1].id), force_bytes("coverage-1"))
        self.assertEqual(inst.created.date, FHIRDate("2014-09-16").date)
        self.assertEqual(inst.created.as_json(), "2014-09-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Policy is currently in-force."))
        self.assertEqual(force_bytes(inst.form.coding[0].code), force_bytes("ELRSP/2017/01"))
        self.assertEqual(force_bytes(inst.form.coding[0].system), force_bytes("http://national.org/form"))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2502"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/eligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("8812342"))
        self.assertTrue(inst.inforce)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].category.coding[0].code), force_bytes("medical"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].allowedMoney.code), force_bytes("USD"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[0].allowedMoney.value, 500000)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].usedMoney.code), force_bytes("USD"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].usedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[0].usedMoney.value, 3748.0)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[1].allowedMoney.code), force_bytes("USD"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[1].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[1].allowedMoney.value, 100)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[1].type.coding[0].code), force_bytes("copay-maximum"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[2].allowedUnsignedInt, 20)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[2].type.coding[0].code), force_bytes("copay-percent"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].subCategory.coding[0].code), force_bytes("30"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].subCategory.coding[0].display), force_bytes("Health Benefit Plan Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].category.coding[0].code), force_bytes("medical"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].financial[0].allowedMoney.code), force_bytes("USD"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[1].financial[0].allowedMoney.value, 15000)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].subCategory.coding[0].code), force_bytes("69"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].subCategory.coding[0].display), force_bytes("Maternity"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].category.coding[0].code), force_bytes("oral"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].financial[0].allowedMoney.code), force_bytes("USD"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[2].financial[0].allowedMoney.value, 2000)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].subCategory.coding[0].code), force_bytes("F3"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].subCategory.coding[0].display), force_bytes("Dental Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].category.coding[0].code), force_bytes("vision"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].description), force_bytes("Vision products and services such as exams, glasses and contatc lenses."))
        self.assertTrue(inst.insurance[0].benefitBalance[3].excluded)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].name), force_bytes("Vision"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].subCategory.coding[0].code), force_bytes("F6"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].subCategory.coding[0].display), force_bytes("Vision Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].code), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].system), force_bytes("http://hl7.org/fhir/remittance-outcome"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the EligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testEligibilityResponse3(self):
        inst = self.instantiate_from("eligibilityresponse-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a EligibilityResponse instance")
        self.implEligibilityResponse3(inst)
        
        js = inst.as_json()
        self.assertEqual("EligibilityResponse", js["resourceType"])
        inst2 = eligibilityresponse.EligibilityResponse(js)
        self.implEligibilityResponse3(inst2)
    
    def implEligibilityResponse3(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Policy is currently in-force."))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2500"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/eligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("881234"))
        self.assertTrue(inst.inforce)
        self.assertEqual(force_bytes(inst.outcome.coding[0].code), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].system), force_bytes("http://hl7.org/fhir/remittance-outcome"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the EligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testEligibilityResponse4(self):
        inst = self.instantiate_from("eligibilityresponse-example-benefits.json")
        self.assertIsNotNone(inst, "Must have instantiated a EligibilityResponse instance")
        self.implEligibilityResponse4(inst)
        
        js = inst.as_json()
        self.assertEqual("EligibilityResponse", js["resourceType"])
        inst2 = eligibilityresponse.EligibilityResponse(js)
        self.implEligibilityResponse4(inst2)
    
    def implEligibilityResponse4(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Policy is currently in-force."))
        self.assertEqual(force_bytes(inst.id), force_bytes("E2501"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/eligibilityresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("881234"))
        self.assertTrue(inst.inforce)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].category.coding[0].code), force_bytes("medical"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].allowedMoney.code), force_bytes("SAR"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[0].allowedMoney.value, 500000)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[1].allowedMoney.code), force_bytes("SAR"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[1].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[1].allowedMoney.value, 100)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[1].type.coding[0].code), force_bytes("copay-maximum"))
        self.assertEqual(inst.insurance[0].benefitBalance[0].financial[2].allowedUnsignedInt, 20)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].financial[2].type.coding[0].code), force_bytes("copay-percent"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].subCategory.coding[0].code), force_bytes("30"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].subCategory.coding[0].display), force_bytes("Health Benefit Plan Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[0].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].category.coding[0].code), force_bytes("medical"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].financial[0].allowedMoney.code), force_bytes("SAR"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[1].financial[0].allowedMoney.value, 15000)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].subCategory.coding[0].code), force_bytes("69"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].subCategory.coding[0].display), force_bytes("Maternity"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[1].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].category.coding[0].code), force_bytes("oral"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].financial[0].allowedMoney.code), force_bytes("SAR"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[2].financial[0].allowedMoney.value, 2000)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].subCategory.coding[0].code), force_bytes("F3"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].subCategory.coding[0].display), force_bytes("Dental Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[2].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].category.coding[0].code), force_bytes("vision"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].financial[0].allowedMoney.code), force_bytes("SAR"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].financial[0].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[3].financial[0].allowedMoney.value, 400)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].financial[0].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].subCategory.coding[0].code), force_bytes("F6"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].subCategory.coding[0].display), force_bytes("Vision Coverage"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].term.coding[0].code), force_bytes("annual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[3].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].category.coding[0].code), force_bytes("vision"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].category.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-category"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].financial[0].allowedString), force_bytes("shared"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].financial[0].type.coding[0].code), force_bytes("room"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].financial[1].allowedMoney.code), force_bytes("SAR"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].financial[1].allowedMoney.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(inst.insurance[0].benefitBalance[4].financial[1].allowedMoney.value, 600)
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].financial[1].type.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].network.coding[0].code), force_bytes("in"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].network.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-network"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].subCategory.coding[0].code), force_bytes("49"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].subCategory.coding[0].display), force_bytes("Hospital Room and Board"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].subCategory.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-subcategory"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].term.coding[0].code), force_bytes("day"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].term.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-term"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].unit.coding[0].code), force_bytes("individual"))
        self.assertEqual(force_bytes(inst.insurance[0].benefitBalance[4].unit.coding[0].system), force_bytes("http://hl7.org/fhir/benefit-unit"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].code), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.outcome.coding[0].system), force_bytes("http://hl7.org/fhir/remittance-outcome"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the EligibilityResponse.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import testreport
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class TestReportTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("TestReport", js["resourceType"])
        return testreport.TestReport(js)
    
    def testTestReport1(self):
        inst = self.instantiate_from("testreport-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestReport instance")
        self.implTestReport1(inst)
        
        js = inst.as_json()
        self.assertEqual("TestReport", js["resourceType"])
        inst2 = testreport.TestReport(js)
        self.implTestReport1(inst2)
    
    def implTestReport1(self, inst):
        self.assertEqual(force_bytes(inst.id), force_bytes("testreport-example"))
        self.assertEqual(force_bytes(inst.identifier.system), force_bytes("urn:ietf:rfc:3986"))
        self.assertEqual(force_bytes(inst.identifier.value), force_bytes("urn:oid:1.3.6.1.4.1.21367.2005.3.7.9878"))
        self.assertEqual(inst.issued.date, FHIRDate("2016-10-07T08:25:34-05:00").date)
        self.assertEqual(inst.issued.as_json(), "2016-10-07T08:25:34-05:00")
        self.assertEqual(force_bytes(inst.name), force_bytes("TestReport Example for TestScript Example"))
        self.assertEqual(force_bytes(inst.participant[0].display), force_bytes("Crucible"))
        self.assertEqual(force_bytes(inst.participant[0].type), force_bytes("test-engine"))
        self.assertEqual(force_bytes(inst.participant[0].uri), force_bytes("http://projectcrucible.org"))
        self.assertEqual(force_bytes(inst.participant[1].display), force_bytes("HealthIntersections STU3"))
        self.assertEqual(force_bytes(inst.participant[1].type), force_bytes("server"))
        self.assertEqual(force_bytes(inst.participant[1].uri), force_bytes("http://fhir3.healthintersections.com.au/open"))
        self.assertEqual(force_bytes(inst.result), force_bytes("pass"))
        self.assertEqual(inst.score, 100.0)
        self.assertEqual(force_bytes(inst.setup.action[0].operation.detail), force_bytes("http://projectcrucible.org/permalink/1"))
        self.assertEqual(force_bytes(inst.setup.action[0].operation.message), force_bytes("DELETE Patient"))
        self.assertEqual(force_bytes(inst.setup.action[0].operation.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.setup.action[1].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/1"))
        self.assertEqual(force_bytes(inst.setup.action[1].assert_fhir.message), force_bytes("HTTP 204"))
        self.assertEqual(force_bytes(inst.setup.action[1].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.setup.action[2].operation.detail), force_bytes("http://projectcrucible.org/permalink/1"))
        self.assertEqual(force_bytes(inst.setup.action[2].operation.message), force_bytes("POST Patient/fixture-patient-create"))
        self.assertEqual(force_bytes(inst.setup.action[2].operation.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.setup.action[3].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/1"))
        self.assertEqual(force_bytes(inst.setup.action[3].assert_fhir.message), force_bytes("HTTP 201"))
        self.assertEqual(force_bytes(inst.setup.action[3].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.status), force_bytes("completed"))
        self.assertEqual(force_bytes(inst.teardown.action[0].operation.detail), force_bytes("http://projectcrucible.org/permalink/3"))
        self.assertEqual(force_bytes(inst.teardown.action[0].operation.message), force_bytes("DELETE Patient/fixture-patient-create."))
        self.assertEqual(force_bytes(inst.teardown.action[0].operation.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[0].operation.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[0].operation.message), force_bytes("GET Patient/fixture-patient-create"))
        self.assertEqual(force_bytes(inst.test[0].action[0].operation.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[1].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[1].assert_fhir.message), force_bytes("HTTP 200"))
        self.assertEqual(force_bytes(inst.test[0].action[1].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[2].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[2].assert_fhir.message), force_bytes("Last-Modified Present"))
        self.assertEqual(force_bytes(inst.test[0].action[2].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[3].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[3].assert_fhir.message), force_bytes("Response is Patient"))
        self.assertEqual(force_bytes(inst.test[0].action[3].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[4].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[4].assert_fhir.message), force_bytes("Response validates"))
        self.assertEqual(force_bytes(inst.test[0].action[4].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[5].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[5].assert_fhir.message), force_bytes("Patient.name.family 'Chalmers'"))
        self.assertEqual(force_bytes(inst.test[0].action[5].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[6].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[6].assert_fhir.message), force_bytes("Patient.name.given 'Peter'"))
        self.assertEqual(force_bytes(inst.test[0].action[6].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[7].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[7].assert_fhir.message), force_bytes("Patient.name.family 'Chalmers'"))
        self.assertEqual(force_bytes(inst.test[0].action[7].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[8].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[8].assert_fhir.message), force_bytes("Patient.name.family 'Chalmers'"))
        self.assertEqual(force_bytes(inst.test[0].action[8].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].action[9].assert_fhir.detail), force_bytes("http://projectcrucible.org/permalink/2"))
        self.assertEqual(force_bytes(inst.test[0].action[9].assert_fhir.message), force_bytes("Patient expected values."))
        self.assertEqual(force_bytes(inst.test[0].action[9].assert_fhir.result), force_bytes("pass"))
        self.assertEqual(force_bytes(inst.test[0].description), force_bytes("Read a Patient and validate response."))
        self.assertEqual(force_bytes(inst.test[0].id), force_bytes("01-ReadPatient"))
        self.assertEqual(force_bytes(inst.test[0].name), force_bytes("Read Patient"))
        self.assertEqual(force_bytes(inst.tester), force_bytes("HL7 Execution Engine"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import task
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class TaskTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Task", js["resourceType"])
        return task.Task(js)
    
    def testTask1(self):
        inst = self.instantiate_from("task-example6.json")
        self.assertIsNotNone(inst, "Must have instantiated a Task instance")
        self.implTask1(inst)
        
        js = inst.as_json()
        self.assertEqual("Task", js["resourceType"])
        inst2 = task.Task(js)
        self.implTask1(inst2)
    
    def implTask1(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-10-31T08:25:05+10:00").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-10-31T08:25:05+10:00")
        self.assertEqual(force_bytes(inst.businessStatus.text), force_bytes("test completed and posted"))
        self.assertEqual(force_bytes(inst.code.text), force_bytes("Lipid Panel"))
        self.assertEqual(force_bytes(inst.description), force_bytes("Create order for getting specimen, Set up inhouse testing,  generate order for any sendouts and submit with specimen"))
        self.assertEqual(inst.executionPeriod.end.date, FHIRDate("2016-10-31T18:45:05+10:00").date)
        self.assertEqual(inst.executionPeriod.end.as_json(), "2016-10-31T18:45:05+10:00")
        self.assertEqual(inst.executionPeriod.start.date, FHIRDate("2016-10-31T08:25:05+10:00").date)
        self.assertEqual(inst.executionPeriod.start.as_json(), "2016-10-31T08:25:05+10:00")
        self.assertEqual(force_bytes(inst.groupIdentifier.system), force_bytes("http:/goodhealth.org/accession/identifiers"))
        self.assertEqual(force_bytes(inst.groupIdentifier.use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.groupIdentifier.value), force_bytes("G20170201-001"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example6"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http:/goodhealth.org/identifiers"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("20170201-001"))
        self.assertEqual(force_bytes(inst.intent), force_bytes("order"))
        self.assertEqual(inst.lastModified.date, FHIRDate("2016-10-31T18:45:05+10:00").date)
        self.assertEqual(inst.lastModified.as_json(), "2016-10-31T18:45:05+10:00")
        self.assertEqual(force_bytes(inst.output[0].type.text), force_bytes("DiagnosticReport generated"))
        self.assertEqual(force_bytes(inst.output[1].type.text), force_bytes("collected specimen"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].code), force_bytes("performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].display), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].system), force_bytes("http://hl7.org/fhir/task-performer-type"))
        self.assertEqual(force_bytes(inst.performerType[0].text), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.priority), force_bytes("routine"))
        self.assertEqual(force_bytes(inst.reason.text), force_bytes("The Task.reason should only be included if there is no Task.focus or if it differs from the reason indicated on the focus"))
        self.assertEqual(inst.restriction.period.end.date, FHIRDate("2016-11-02T09:45:05+10:00").date)
        self.assertEqual(inst.restriction.period.end.as_json(), "2016-11-02T09:45:05+10:00")
        self.assertEqual(inst.restriction.repetitions, 1)
        self.assertEqual(force_bytes(inst.status), force_bytes("completed"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testTask2(self):
        inst = self.instantiate_from("task-example2.json")
        self.assertIsNotNone(inst, "Must have instantiated a Task instance")
        self.implTask2(inst)
        
        js = inst.as_json()
        self.assertEqual("Task", js["resourceType"])
        inst2 = task.Task(js)
        self.implTask2(inst2)
    
    def implTask2(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-10-31T08:45:05+10:00").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-10-31T08:45:05+10:00")
        self.assertEqual(force_bytes(inst.businessStatus.text), force_bytes("waiting for patient"))
        self.assertEqual(force_bytes(inst.code.text), force_bytes("Specimen Collection"))
        self.assertEqual(inst.executionPeriod.start.date, FHIRDate("2016-10-31T08:45:05+10:00").date)
        self.assertEqual(inst.executionPeriod.start.as_json(), "2016-10-31T08:45:05+10:00")
        self.assertEqual(force_bytes(inst.groupIdentifier.system), force_bytes("http:/goodhealth.org/accession/identifiers"))
        self.assertEqual(force_bytes(inst.groupIdentifier.use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.groupIdentifier.value), force_bytes("G20170201-001"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example2"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http:/goodhealth.org/identifiers"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("20170201-002"))
        self.assertEqual(force_bytes(inst.intent), force_bytes("filler-order"))
        self.assertEqual(inst.lastModified.date, FHIRDate("2016-10-31T09:45:05+10:00").date)
        self.assertEqual(inst.lastModified.as_json(), "2016-10-31T09:45:05+10:00")
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].code), force_bytes("performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].display), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].system), force_bytes("http://hl7.org/fhir/task-performer-type"))
        self.assertEqual(force_bytes(inst.performerType[0].text), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.priority), force_bytes("routine"))
        self.assertEqual(inst.restriction.period.end.date, FHIRDate("2016-11-01T09:45:05+10:00").date)
        self.assertEqual(inst.restriction.period.end.as_json(), "2016-11-01T09:45:05+10:00")
        self.assertEqual(inst.restriction.repetitions, 1)
        self.assertEqual(force_bytes(inst.status), force_bytes("accepted"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testTask3(self):
        inst = self.instantiate_from("task-example5.json")
        self.assertIsNotNone(inst, "Must have instantiated a Task instance")
        self.implTask3(inst)
        
        js = inst.as_json()
        self.assertEqual("Task", js["resourceType"])
        inst2 = task.Task(js)
        self.implTask3(inst2)
    
    def implTask3(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-10-31T08:25:05+10:00").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-10-31T08:25:05+10:00")
        self.assertEqual(force_bytes(inst.businessStatus.text), force_bytes("specimen received, test in progress"))
        self.assertEqual(force_bytes(inst.code.text), force_bytes("Lipid Panel"))
        self.assertEqual(force_bytes(inst.description), force_bytes("Create order for getting specimen, Set up inhouse testing,  generate order for any sendouts and submit with specimen"))
        self.assertEqual(inst.executionPeriod.start.date, FHIRDate("2016-10-31T08:25:05+10:00").date)
        self.assertEqual(inst.executionPeriod.start.as_json(), "2016-10-31T08:25:05+10:00")
        self.assertEqual(force_bytes(inst.groupIdentifier.system), force_bytes("http:/goodhealth.org/accession/identifiers"))
        self.assertEqual(force_bytes(inst.groupIdentifier.use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.groupIdentifier.value), force_bytes("G20170201-001"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example5"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http:/goodhealth.org/identifiers"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("20170201-001"))
        self.assertEqual(force_bytes(inst.intent), force_bytes("order"))
        self.assertEqual(inst.lastModified.date, FHIRDate("2016-10-31T16:45:05+10:00").date)
        self.assertEqual(inst.lastModified.as_json(), "2016-10-31T16:45:05+10:00")
        self.assertEqual(force_bytes(inst.output[0].type.text), force_bytes("collected specimen"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].code), force_bytes("performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].display), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].system), force_bytes("http://hl7.org/fhir/task-performer-type"))
        self.assertEqual(force_bytes(inst.performerType[0].text), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.priority), force_bytes("routine"))
        self.assertEqual(force_bytes(inst.reason.text), force_bytes("The Task.reason should only be included if there is no Task.focus or if it differs from the reason indicated on the focus"))
        self.assertEqual(inst.restriction.period.end.date, FHIRDate("2016-11-02T09:45:05+10:00").date)
        self.assertEqual(inst.restriction.period.end.as_json(), "2016-11-02T09:45:05+10:00")
        self.assertEqual(inst.restriction.repetitions, 1)
        self.assertEqual(force_bytes(inst.status), force_bytes("in-progress"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testTask4(self):
        inst = self.instantiate_from("task-example1.json")
        self.assertIsNotNone(inst, "Must have instantiated a Task instance")
        self.implTask4(inst)
        
        js = inst.as_json()
        self.assertEqual("Task", js["resourceType"])
        inst2 = task.Task(js)
        self.implTask4(inst2)
    
    def implTask4(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-10-31T08:25:05+10:00").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-10-31T08:25:05+10:00")
        self.assertEqual(force_bytes(inst.businessStatus.text), force_bytes("waiting for specimen"))
        self.assertEqual(force_bytes(inst.code.text), force_bytes("Lipid Panel"))
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("signature"))
        self.assertEqual(force_bytes(inst.description), force_bytes("Create order for getting specimen, Set up inhouse testing,  generate order for any sendouts and submit with specimen"))
        self.assertEqual(inst.executionPeriod.start.date, FHIRDate("2016-10-31T08:25:05+10:00").date)
        self.assertEqual(inst.executionPeriod.start.as_json(), "2016-10-31T08:25:05+10:00")
        self.assertEqual(force_bytes(inst.groupIdentifier.system), force_bytes("http:/goodhealth.org/accession/identifiers"))
        self.assertEqual(force_bytes(inst.groupIdentifier.use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.groupIdentifier.value), force_bytes("G20170201-001"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example1"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http:/goodhealth.org/identifiers"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("20170201-001"))
        self.assertEqual(force_bytes(inst.intent), force_bytes("order"))
        self.assertEqual(inst.lastModified.date, FHIRDate("2016-10-31T09:45:05+10:00").date)
        self.assertEqual(inst.lastModified.as_json(), "2016-10-31T09:45:05+10:00")
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].code), force_bytes("performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].display), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].system), force_bytes("http://hl7.org/fhir/task-performer-type"))
        self.assertEqual(force_bytes(inst.performerType[0].text), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.priority), force_bytes("routine"))
        self.assertEqual(force_bytes(inst.reason.text), force_bytes("The Task.reason should only be included if there is no Task.focus or if it differs from the reason indicated on the focus"))
        self.assertEqual(inst.restriction.period.end.date, FHIRDate("2016-11-02T09:45:05+10:00").date)
        self.assertEqual(inst.restriction.period.end.as_json(), "2016-11-02T09:45:05+10:00")
        self.assertEqual(inst.restriction.repetitions, 1)
        self.assertEqual(force_bytes(inst.status), force_bytes("in-progress"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testTask5(self):
        inst = self.instantiate_from("task-example4.json")
        self.assertIsNotNone(inst, "Must have instantiated a Task instance")
        self.implTask5(inst)
        
        js = inst.as_json()
        self.assertEqual("Task", js["resourceType"])
        inst2 = task.Task(js)
        self.implTask5(inst2)
    
    def implTask5(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-10-31T08:45:05+10:00").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-10-31T08:45:05+10:00")
        self.assertEqual(force_bytes(inst.code.text), force_bytes("Specimen Collection"))
        self.assertEqual(inst.executionPeriod.end.date, FHIRDate("2016-10-31T14:45:05+10:00").date)
        self.assertEqual(inst.executionPeriod.end.as_json(), "2016-10-31T14:45:05+10:00")
        self.assertEqual(inst.executionPeriod.start.date, FHIRDate("2016-10-31T08:45:05+10:00").date)
        self.assertEqual(inst.executionPeriod.start.as_json(), "2016-10-31T08:45:05+10:00")
        self.assertEqual(force_bytes(inst.groupIdentifier.system), force_bytes("http:/goodhealth.org/accession/identifiers"))
        self.assertEqual(force_bytes(inst.groupIdentifier.use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.groupIdentifier.value), force_bytes("G20170201-001"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example4"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http:/goodhealth.org/identifiers"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("20170201-002"))
        self.assertEqual(force_bytes(inst.intent), force_bytes("filler-order"))
        self.assertEqual(inst.lastModified.date, FHIRDate("2016-10-31T09:45:05+10:00").date)
        self.assertEqual(inst.lastModified.as_json(), "2016-10-31T09:45:05+10:00")
        self.assertEqual(force_bytes(inst.output[0].type.text), force_bytes("collected specimen"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].code), force_bytes("performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].display), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.performerType[0].coding[0].system), force_bytes("http://hl7.org/fhir/task-performer-type"))
        self.assertEqual(force_bytes(inst.performerType[0].text), force_bytes("Performer"))
        self.assertEqual(force_bytes(inst.priority), force_bytes("routine"))
        self.assertEqual(inst.restriction.period.end.date, FHIRDate("2016-11-01T09:45:05+10:00").date)
        self.assertEqual(inst.restriction.period.end.as_json(), "2016-11-01T09:45:05+10:00")
        self.assertEqual(inst.restriction.repetitions, 1)
        self.assertEqual(force_bytes(inst.status), force_bytes("completed"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testTask6(self):
        inst = self.instantiate_from("task-example3.json")
        self.assertIsNotNone(inst, "Must have instantiated a Task instance")
        self.implTask6(inst)
        
        js = inst.as_json()
        self.assertEqual("Task", js["resourceType"])
        inst2 = task.Task(js)
        self.implTask6(inst2)
    
    def implTask6(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-03-10T22:39:32-04:00").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-03-10T22:39:32-04:00")
        self.assertEqual(force_bytes(inst.code.text), force_bytes("Refill Request"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example3"))
        self.assertEqual(force_bytes(inst.intent), force_bytes("order"))
        self.assertEqual(inst.lastModified.date, FHIRDate("2016-03-10T22:39:32-04:00").date)
        self.assertEqual(inst.lastModified.as_json(), "2016-03-10T22:39:32-04:00")
        self.assertEqual(force_bytes(inst.status), force_bytes("draft"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import specimen
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class SpecimenTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Specimen", js["resourceType"])
        return specimen.Specimen(js)
    
    def testSpecimen1(self):
        inst = self.instantiate_from("specimen-example-serum.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen1(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen1(inst2)
    
    def implSpecimen1(self, inst):
        self.assertEqual(force_bytes(inst.accessionIdentifier.system), force_bytes("http://acme.com/labs/accession-ids"))
        self.assertEqual(force_bytes(inst.accessionIdentifier.value), force_bytes("20150816-00124"))
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2015-08-16T06:40:17Z").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2015-08-16T06:40:17Z")
        self.assertEqual(force_bytes(inst.container[0].type.coding[0].code), force_bytes("SST"))
        self.assertEqual(force_bytes(inst.container[0].type.coding[0].display), force_bytes("Serum Separator Tube"))
        self.assertEqual(force_bytes(inst.container[0].type.coding[0].system), force_bytes("http://acme.com/labs"))
        self.assertEqual(force_bytes(inst.id), force_bytes("sst"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("119364003"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Serum sample"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://snomed.info/sct"))
    
    def testSpecimen2(self):
        inst = self.instantiate_from("specimen-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen2(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen2(inst2)
    
    def implSpecimen2(self, inst):
        self.assertEqual(force_bytes(inst.accessionIdentifier.system), force_bytes("http://lab.acme.org/specimens/2011"))
        self.assertEqual(force_bytes(inst.accessionIdentifier.value), force_bytes("X352356"))
        self.assertEqual(force_bytes(inst.collection.bodySite.coding[0].code), force_bytes("49852007"))
        self.assertEqual(force_bytes(inst.collection.bodySite.coding[0].display), force_bytes("Structure of median cubital vein (body structure)"))
        self.assertEqual(force_bytes(inst.collection.bodySite.coding[0].system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.collection.bodySite.text), force_bytes("Right median cubital vein"))
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2011-05-30T06:15:00Z").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2011-05-30T06:15:00Z")
        self.assertEqual(force_bytes(inst.collection.method.coding[0].code), force_bytes("LNV"))
        self.assertEqual(force_bytes(inst.collection.method.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0488"))
        self.assertEqual(force_bytes(inst.collection.quantity.unit), force_bytes("mL"))
        self.assertEqual(inst.collection.quantity.value, 6)
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("hep"))
        self.assertEqual(force_bytes(inst.container[0].capacity.unit), force_bytes("mL"))
        self.assertEqual(inst.container[0].capacity.value, 10)
        self.assertEqual(force_bytes(inst.container[0].description), force_bytes("Green Gel tube"))
        self.assertEqual(force_bytes(inst.container[0].identifier[0].value), force_bytes("48736-15394-75465"))
        self.assertEqual(force_bytes(inst.container[0].specimenQuantity.unit), force_bytes("mL"))
        self.assertEqual(inst.container[0].specimenQuantity.value, 6)
        self.assertEqual(force_bytes(inst.container[0].type.text), force_bytes("Vacutainer"))
        self.assertEqual(force_bytes(inst.id), force_bytes("101"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://ehr.acme.org/identifiers/collections"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("23234352356"))
        self.assertEqual(force_bytes(inst.note[0].text), force_bytes("Specimen is grossly lipemic"))
        self.assertEqual(inst.receivedTime.date, FHIRDate("2011-03-04T07:03:00Z").date)
        self.assertEqual(inst.receivedTime.as_json(), "2011-03-04T07:03:00Z")
        self.assertEqual(force_bytes(inst.status), force_bytes("available"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("122555007"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Venous blood specimen"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://snomed.info/sct"))
    
    def testSpecimen3(self):
        inst = self.instantiate_from("specimen-example-urine.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen3(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen3(inst2)
    
    def implSpecimen3(self, inst):
        self.assertEqual(force_bytes(inst.accessionIdentifier.system), force_bytes("http://lab.acme.org/specimens/2015"))
        self.assertEqual(force_bytes(inst.accessionIdentifier.value), force_bytes("X352356"))
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2015-08-18T07:03:00Z").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2015-08-18T07:03:00Z")
        self.assertEqual(force_bytes(inst.container[0].capacity.unit), force_bytes("mls"))
        self.assertEqual(inst.container[0].capacity.value, 50)
        self.assertEqual(force_bytes(inst.container[0].specimenQuantity.unit), force_bytes("mls"))
        self.assertEqual(inst.container[0].specimenQuantity.value, 10)
        self.assertEqual(force_bytes(inst.container[0].type.text), force_bytes("Non-sterile specimen container"))
        self.assertEqual(force_bytes(inst.id), force_bytes("vma-urine"))
        self.assertEqual(force_bytes(inst.processing[0].description), force_bytes("Acidify to pH < 3.0 with 6 N HCl."))
        self.assertEqual(force_bytes(inst.processing[0].procedure.coding[0].code), force_bytes("ACID"))
        self.assertEqual(force_bytes(inst.processing[0].procedure.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0373"))
        self.assertEqual(inst.processing[0].timeDateTime.date, FHIRDate("2015-08-18T08:10:00Z").date)
        self.assertEqual(inst.processing[0].timeDateTime.as_json(), "2015-08-18T08:10:00Z")
        self.assertEqual(inst.receivedTime.date, FHIRDate("2015-08-18T07:03:00Z").date)
        self.assertEqual(inst.receivedTime.as_json(), "2015-08-18T07:03:00Z")
        self.assertEqual(force_bytes(inst.status), force_bytes("available"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("RANDU"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Urine, Random"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0487"))
    
    def testSpecimen4(self):
        inst = self.instantiate_from("specimen-example-isolate.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen4(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen4(inst2)
    
    def implSpecimen4(self, inst):
        self.assertEqual(force_bytes(inst.accessionIdentifier.system), force_bytes("http://lab.acme.org/specimens/2011"))
        self.assertEqual(force_bytes(inst.accessionIdentifier.value), force_bytes("X352356-ISO1"))
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2015-08-16T07:03:00Z").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2015-08-16T07:03:00Z")
        self.assertEqual(force_bytes(inst.collection.method.coding[0].code), force_bytes("BAP"))
        self.assertEqual(force_bytes(inst.collection.method.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0488"))
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("stool"))
        self.assertEqual(force_bytes(inst.id), force_bytes("isolate"))
        self.assertEqual(force_bytes(inst.note[0].text), force_bytes("Patient dropped off specimen"))
        self.assertEqual(inst.receivedTime.date, FHIRDate("2015-08-18T07:03:00Z").date)
        self.assertEqual(inst.receivedTime.as_json(), "2015-08-18T07:03:00Z")
        self.assertEqual(force_bytes(inst.status), force_bytes("available"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("429951000124103"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Bacterial isolate specimen"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://snomed.info/sct"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Medication) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class Medication(domainresource.DomainResource):
    """ Definition of a Medication.
    
    This resource is primarily used for the identification and definition of a
    medication. It covers the ingredients and the packaging for a medication.
    """
    
    resource_type = "Medication"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.code = None
        """ Codes that identify this medication.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.form = None
        """ powder | tablets | capsule +.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.image = None
        """ Picture of the medication.
        List of `Attachment` items (represented as `dict` in JSON). """
        
        self.ingredient = None
        """ Active or inactive ingredient.
        List of `MedicationIngredient` items (represented as `dict` in JSON). """
        
        self.isBrand = None
        """ True if a brand.
        Type `bool`. """
        
        self.isOverTheCounter = None
        """ True if medication does not require a prescription.
        Type `bool`. """
        
        self.manufacturer = None
        """ Manufacturer of the item.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """
        
        self.package = None
        """ Details about packaged medications.
        Type `MedicationPackage` (represented as `dict` in JSON). """
        
        self.status = None
        """ active | inactive | entered-in-error.
        Type `str`. """
        
        super(Medication, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Medication, self).elementProperties()
        js.extend([
            ("code", "code", codeableconcept.CodeableConcept, False, None, False),
            ("form", "form", codeableconcept.CodeableConcept, False, None, False),
            ("image", "image", attachment.Attachment, True, None, False),
            ("ingredient", "ingredient", MedicationIngredient, True, None, False),
            ("isBrand", "isBrand", bool, False, None, False),
            ("isOverTheCounter", "isOverTheCounter", bool, False, None, False),
            ("manufacturer", "manufacturer", fhirreference.FHIRReference, False, None, False),
            ("package", "package", MedicationPackage, False, None, False),
            ("status", "status", str, False, None, False),
        ])
        return js


from . import backboneelement

class MedicationIngredient(backboneelement.BackboneElement):
    """ Active or inactive ingredient.
    
    Identifies a particular constituent of interest in the product.
    """
    
    resource_type = "MedicationIngredient"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.amount = None
        """ Quantity of ingredient present.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.isActive = None
        """ Active ingredient indicator.
        Type `bool`. """
        
        self.itemCodeableConcept = None
        """ The product contained.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.itemReference = None
        """ The product contained.
        Type `FHIRReference` referencing `Substance, Medication` (represented as `dict` in JSON). """
        
        super(MedicationIngredient, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationIngredient, self).elementProperties()
        js.extend([
            ("amount", "amount", ratio.Ratio, False, None, False),
            ("isActive", "isActive", bool, False, None, False),
            ("itemCodeableConcept", "itemCodeableConcept", codeableconcept.CodeableConcept, False, "item", True),
            ("itemReference", "itemReference", fhirreference.FHIRReference, False, "item", True),
        ])
        return js


class MedicationPackage(backboneelement.BackboneElement):
    """ Details about packaged medications.
    
    Information that only applies to packages (not products).
    """
    
    resource_type = "MedicationPackage"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.batch = None
        """ Identifies a single production run.
        List of `MedicationPackageBatch` items (represented as `dict` in JSON). """
        
        self.container = None
        """ E.g. box, vial, blister-pack.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.content = None
        """ What is  in the package.
        List of `MedicationPackageContent` items (represented as `dict` in JSON). """
        
        super(MedicationPackage, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationPackage, self).elementProperties()
        js.extend([
            ("batch", "batch", MedicationPackageBatch, True, None, False),
            ("container", "container", codeableconcept.CodeableConcept, False, None, False),
            ("content", "content", MedicationPackageContent, True, None, False),
        ])
        return js


class MedicationPackageBatch(backboneelement.BackboneElement):
    """ Identifies a single production run.
    
    Information about a group of medication produced or packaged from one
    production run.
    """
    
    resource_type = "MedicationPackageBatch"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.expirationDate = None
        """ When batch will expire.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.lotNumber = None
        """ Identifier assigned to batch.
        Type `str`. """
        
        super(MedicationPackageBatch, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationPackageBatch, self).elementProperties()
        js.extend([
            ("expirationDate", "expirationDate", fhirdate.FHIRDate, False, None, False),
            ("lotNumber", "lotNumber", str, False, None, False),
        ])
        return js


class MedicationPackageContent(backboneelement.BackboneElement):
    """ What is  in the package.
    
    A set of components that go to make up the described item.
    """
    
    resource_type = "MedicationPackageContent"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.amount = None
        """ Quantity present in the package.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.itemCodeableConcept = None
        """ The item in the package.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.itemReference = None
        """ The item in the package.
        Type `FHIRReference` referencing `Medication` (represented as `dict` in JSON). """
        
        super(MedicationPackageContent, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationPackageContent, self).elementProperties()
        js.extend([
            ("amount", "amount", quantity.Quantity, False, None, False),
            ("itemCodeableConcept", "itemCodeableConcept", codeableconcept.CodeableConcept, False, "item", True),
            ("itemReference", "itemReference", fhirreference.FHIRReference, False, "item", True),
        ])
        return js


import sys
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']
try:
    from . import ratio
except ImportError:
    ratio = sys.modules[__package__ + '.ratio']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import messageheader
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class MessageHeaderTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("MessageHeader", js["resourceType"])
        return messageheader.MessageHeader(js)
    
    def testMessageHeader1(self):
        inst = self.instantiate_from("messageheader-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a MessageHeader instance")
        self.implMessageHeader1(inst)
        
        js = inst.as_json()
        self.assertEqual("MessageHeader", js["resourceType"])
        inst2 = messageheader.MessageHeader(js)
        self.implMessageHeader1(inst2)
    
    def implMessageHeader1(self, inst):
        self.assertEqual(force_bytes(inst.destination[0].endpoint), force_bytes("llp:10.11.12.14:5432"))
        self.assertEqual(force_bytes(inst.destination[0].name), force_bytes("Acme Message Gateway"))
        self.assertEqual(force_bytes(inst.event.code), force_bytes("admin-notify"))
        self.assertEqual(force_bytes(inst.event.system), force_bytes("http://hl7.org/fhir/message-events"))
        self.assertEqual(force_bytes(inst.id), force_bytes("1cbdfb97-5859-48a4-8301-d54eab818d68"))
        self.assertEqual(force_bytes(inst.reason.coding[0].code), force_bytes("admit"))
        self.assertEqual(force_bytes(inst.reason.coding[0].system), force_bytes("http://hl7.org/fhir/message-reasons-encounter"))
        self.assertEqual(force_bytes(inst.response.code), force_bytes("ok"))
        self.assertEqual(force_bytes(inst.response.identifier), force_bytes("5015fe84-8e76-4526-89d8-44b322e8d4fb"))
        self.assertEqual(force_bytes(inst.source.contact.system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.source.contact.value), force_bytes("+1 (555) 123 4567"))
        self.assertEqual(force_bytes(inst.source.endpoint), force_bytes("llp:10.11.12.13:5432"))
        self.assertEqual(force_bytes(inst.source.name), force_bytes("Acme Central Patient Registry"))
        self.assertEqual(force_bytes(inst.source.software), force_bytes("FooBar Patient Manager"))
        self.assertEqual(force_bytes(inst.source.version), force_bytes("3.1.45.AABB"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(inst.timestamp.date, FHIRDate("2012-01-04T09:10:14Z").date)
        self.assertEqual(inst.timestamp.as_json(), "2012-01-04T09:10:14Z")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import location
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class LocationTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Location", js["resourceType"])
        return location.Location(js)
    
    def testLocation1(self):
        inst = self.instantiate_from("location-example-room.json")
        self.assertIsNotNone(inst, "Must have instantiated a Location instance")
        self.implLocation1(inst)
        
        js = inst.as_json()
        self.assertEqual("Location", js["resourceType"])
        inst2 = location.Location(js)
        self.implLocation1(inst2)
    
    def implLocation1(self, inst):
        self.assertEqual(force_bytes(inst.alias[0]), force_bytes("South Wing OR 5"))
        self.assertEqual(force_bytes(inst.alias[1]), force_bytes("Main Wing OR 2"))
        self.assertEqual(force_bytes(inst.description), force_bytes("Old South Wing, Neuro Radiology Operation Room 1 on second floor"))
        self.assertEqual(force_bytes(inst.id), force_bytes("2"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("B1-S.F2.1.00"))
        self.assertEqual(force_bytes(inst.mode), force_bytes("instance"))
        self.assertEqual(force_bytes(inst.name), force_bytes("South Wing Neuro OR 1"))
        self.assertEqual(force_bytes(inst.operationalStatus.code), force_bytes("H"))
        self.assertEqual(force_bytes(inst.operationalStatus.display), force_bytes("Housekeeping"))
        self.assertEqual(force_bytes(inst.operationalStatus.system), force_bytes("http://hl7.org/fhir/v2/0116"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].code), force_bytes("ro"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].display), force_bytes("Room"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].system), force_bytes("http://hl7.org/fhir/location-physical-type"))
        self.assertEqual(force_bytes(inst.status), force_bytes("suspended"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("2329"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Burgers UMC, South Wing, second floor, Neuro Radiology Operation Room 1</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("RNEU"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Neuroradiology unit"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
    
    def testLocation2(self):
        inst = self.instantiate_from("location-example-hl7hq.json")
        self.assertIsNotNone(inst, "Must have instantiated a Location instance")
        self.implLocation2(inst)
        
        js = inst.as_json()
        self.assertEqual("Location", js["resourceType"])
        inst2 = location.Location(js)
        self.implLocation2(inst2)
    
    def implLocation2(self, inst):
        self.assertEqual(force_bytes(inst.address.city), force_bytes("Ann Arbor"))
        self.assertEqual(force_bytes(inst.address.country), force_bytes("USA"))
        self.assertEqual(force_bytes(inst.address.line[0]), force_bytes("3300 Washtenaw Avenue, Suite 227"))
        self.assertEqual(force_bytes(inst.address.postalCode), force_bytes("48104"))
        self.assertEqual(force_bytes(inst.address.state), force_bytes("MI"))
        self.assertEqual(force_bytes(inst.description), force_bytes("HL7 Headquarters"))
        self.assertEqual(force_bytes(inst.id), force_bytes("hl7"))
        self.assertEqual(force_bytes(inst.mode), force_bytes("instance"))
        self.assertEqual(force_bytes(inst.name), force_bytes("Health Level Seven International"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].code), force_bytes("bu"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].display), force_bytes("Building"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].system), force_bytes("http://hl7.org/fhir/location-physical-type"))
        self.assertEqual(inst.position.latitude, -83.69471)
        self.assertEqual(inst.position.longitude, 42.2565)
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("(+1) 734-677-7777"))
        self.assertEqual(force_bytes(inst.telecom[1].system), force_bytes("fax"))
        self.assertEqual(force_bytes(inst.telecom[1].value), force_bytes("(+1) 734-677-6622"))
        self.assertEqual(force_bytes(inst.telecom[2].system), force_bytes("email"))
        self.assertEqual(force_bytes(inst.telecom[2].value), force_bytes("hq@HL7.org"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("SLEEP"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Sleep disorders unit"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
    
    def testLocation3(self):
        inst = self.instantiate_from("location-example-ambulance.json")
        self.assertIsNotNone(inst, "Must have instantiated a Location instance")
        self.implLocation3(inst)
        
        js = inst.as_json()
        self.assertEqual("Location", js["resourceType"])
        inst2 = location.Location(js)
        self.implLocation3(inst2)
    
    def implLocation3(self, inst):
        self.assertEqual(force_bytes(inst.description), force_bytes("Ambulance provided by Burgers University Medical Center"))
        self.assertEqual(force_bytes(inst.id), force_bytes("amb"))
        self.assertEqual(force_bytes(inst.mode), force_bytes("kind"))
        self.assertEqual(force_bytes(inst.name), force_bytes("BUMC Ambulance"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].code), force_bytes("ve"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].display), force_bytes("Vehicle"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].system), force_bytes("http://hl7.org/fhir/location-physical-type"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("mobile"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("2329"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Mobile Clinic</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("AMB"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Ambulance"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
    
    def testLocation4(self):
        inst = self.instantiate_from("location-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a Location instance")
        self.implLocation4(inst)
        
        js = inst.as_json()
        self.assertEqual("Location", js["resourceType"])
        inst2 = location.Location(js)
        self.implLocation4(inst2)
    
    def implLocation4(self, inst):
        self.assertEqual(force_bytes(inst.address.city), force_bytes("Den Burg"))
        self.assertEqual(force_bytes(inst.address.country), force_bytes("NLD"))
        self.assertEqual(force_bytes(inst.address.line[0]), force_bytes("Galapagosweg 91, Building A"))
        self.assertEqual(force_bytes(inst.address.postalCode), force_bytes("9105 PZ"))
        self.assertEqual(force_bytes(inst.address.use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.description), force_bytes("Second floor of the Old South Wing, formerly in use by Psychiatry"))
        self.assertEqual(force_bytes(inst.extension[0].url), force_bytes("http://hl7.org/fhir/StructureDefinition/location-alias"))
        self.assertEqual(force_bytes(inst.extension[0].valueString), force_bytes("Burgers University Medical Center, South Wing, second floor"))
        self.assertEqual(force_bytes(inst.extension[1].url), force_bytes("http://hl7.org/fhir/StructureDefinition/location-alias"))
        self.assertEqual(force_bytes(inst.extension[1].valueString), force_bytes("BU MC, SW, F2"))
        self.assertEqual(force_bytes(inst.id), force_bytes("1"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("B1-S.F2"))
        self.assertEqual(force_bytes(inst.mode), force_bytes("instance"))
        self.assertEqual(force_bytes(inst.name), force_bytes("South Wing, second floor"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].code), force_bytes("wi"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].display), force_bytes("Wing"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].system), force_bytes("http://hl7.org/fhir/location-physical-type"))
        self.assertEqual(inst.position.altitude, 0)
        self.assertEqual(inst.position.latitude, 42.25475478)
        self.assertEqual(inst.position.longitude, -83.6945691)
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("2328"))
        self.assertEqual(force_bytes(inst.telecom[1].system), force_bytes("fax"))
        self.assertEqual(force_bytes(inst.telecom[1].use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.telecom[1].value), force_bytes("2329"))
        self.assertEqual(force_bytes(inst.telecom[2].system), force_bytes("email"))
        self.assertEqual(force_bytes(inst.telecom[2].value), force_bytes("second wing admissions"))
        self.assertEqual(force_bytes(inst.telecom[3].system), force_bytes("url"))
        self.assertEqual(force_bytes(inst.telecom[3].use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.telecom[3].value), force_bytes("http://sampleorg.com/southwing"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Burgers UMC, South Wing, second floor</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testLocation5(self):
        inst = self.instantiate_from("location-example-patients-home.json")
        self.assertIsNotNone(inst, "Must have instantiated a Location instance")
        self.implLocation5(inst)
        
        js = inst.as_json()
        self.assertEqual("Location", js["resourceType"])
        inst2 = location.Location(js)
        self.implLocation5(inst2)
    
    def implLocation5(self, inst):
        self.assertEqual(force_bytes(inst.description), force_bytes("Patient's Home"))
        self.assertEqual(force_bytes(inst.id), force_bytes("ph"))
        self.assertEqual(force_bytes(inst.mode), force_bytes("kind"))
        self.assertEqual(force_bytes(inst.name), force_bytes("Patient's Home"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].code), force_bytes("ho"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].display), force_bytes("House"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].system), force_bytes("http://hl7.org/fhir/location-physical-type"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Patient's Home</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("PTRES"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Patient's Residence"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
    
    def testLocation6(self):
        inst = self.instantiate_from("location-example-ukpharmacy.json")
        self.assertIsNotNone(inst, "Must have instantiated a Location instance")
        self.implLocation6(inst)
        
        js = inst.as_json()
        self.assertEqual("Location", js["resourceType"])
        inst2 = location.Location(js)
        self.implLocation6(inst2)
    
    def implLocation6(self, inst):
        self.assertEqual(force_bytes(inst.description), force_bytes("All Pharmacies in the United Kingdom covered by the National Pharmacy Association"))
        self.assertEqual(force_bytes(inst.id), force_bytes("ukp"))
        self.assertEqual(force_bytes(inst.mode), force_bytes("kind"))
        self.assertEqual(force_bytes(inst.name), force_bytes("UK Pharmacies"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].code), force_bytes("jdn"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].display), force_bytes("Jurisdiction"))
        self.assertEqual(force_bytes(inst.physicalType.coding[0].system), force_bytes("http://hl7.org/fhir/location-physical-type"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">UK Pharmacies</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("PHARM"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Pharmacy"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/OperationDefinition) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class OperationDefinition(domainresource.DomainResource):
    """ Definition of an operation or a named query.
    
    A formal computable definition of an operation (on the RESTful interface)
    or a named query (using the search interaction).
    """
    
    resource_type = "OperationDefinition"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.base = None
        """ Marks this as a profile of the base.
        Type `FHIRReference` referencing `OperationDefinition` (represented as `dict` in JSON). """
        
        self.code = None
        """ Name used to invoke the operation.
        Type `str`. """
        
        self.comment = None
        """ Additional information about use.
        Type `str`. """
        
        self.contact = None
        """ Contact details for the publisher.
        List of `ContactDetail` items (represented as `dict` in JSON). """
        
        self.date = None
        """ Date this was last changed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.description = None
        """ Natural language description of the operation definition.
        Type `str`. """
        
        self.experimental = None
        """ For testing purposes, not real usage.
        Type `bool`. """
        
        self.idempotent = None
        """ Whether content is unchanged by the operation.
        Type `bool`. """
        
        self.instance = None
        """ Invoke on an instance?.
        Type `bool`. """
        
        self.jurisdiction = None
        """ Intended jurisdiction for operation definition (if applicable).
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.kind = None
        """ operation | query.
        Type `str`. """
        
        self.name = None
        """ Name for this operation definition (computer friendly).
        Type `str`. """
        
        self.overload = None
        """ Define overloaded variants for when  generating code.
        List of `OperationDefinitionOverload` items (represented as `dict` in JSON). """
        
        self.parameter = None
        """ Parameters for the operation/query.
        List of `OperationDefinitionParameter` items (represented as `dict` in JSON). """
        
        self.publisher = None
        """ Name of the publisher (organization or individual).
        Type `str`. """
        
        self.purpose = None
        """ Why this operation definition is defined.
        Type `str`. """
        
        self.resource = None
        """ Types this operation applies to.
        List of `str` items. """
        
        self.status = None
        """ draft | active | retired | unknown.
        Type `str`. """
        
        self.system = None
        """ Invoke at the system level?.
        Type `bool`. """
        
        self.type = None
        """ Invole at the type level?.
        Type `bool`. """
        
        self.url = None
        """ Logical URI to reference this operation definition (globally
        unique).
        Type `str`. """
        
        self.useContext = None
        """ Context the content is intended to support.
        List of `UsageContext` items (represented as `dict` in JSON). """
        
        self.version = None
        """ Business version of the operation definition.
        Type `str`. """
        
        super(OperationDefinition, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(OperationDefinition, self).elementProperties()
        js.extend([
            ("base", "base", fhirreference.FHIRReference, False, None, False),
            ("code", "code", str, False, None, True),
            ("comment", "comment", str, False, None, False),
            ("contact", "contact", contactdetail.ContactDetail, True, None, False),
            ("date", "date", fhirdate.FHIRDate, False, None, False),
            ("description", "description", str, False, None, False),
            ("experimental", "experimental", bool, False, None, False),
            ("idempotent", "idempotent", bool, False, None, False),
            ("instance", "instance", bool, False, None, True),
            ("jurisdiction", "jurisdiction", codeableconcept.CodeableConcept, True, None, False),
            ("kind", "kind", str, False, None, True),
            ("name", "name", str, False, None, True),
            ("overload", "overload", OperationDefinitionOverload, True, None, False),
            ("parameter", "parameter", OperationDefinitionParameter, True, None, False),
            ("publisher", "publisher", str, False, None, False),
            ("purpose", "purpose", str, False, None, False),
            ("resource", "resource", str, True, None, False),
            ("status", "status", str, False, None, True),
            ("system", "system", bool, False, None, True),
            ("type", "type", bool, False, None, True),
            ("url", "url", str, False, None, False),
            ("useContext", "useContext", usagecontext.UsageContext, True, None, False),
            ("version", "version", str, False, None, False),
        ])
        return js


from . import backboneelement

class OperationDefinitionOverload(backboneelement.BackboneElement):
    """ Define overloaded variants for when  generating code.
    
    Defines an appropriate combination of parameters to use when invoking this
    operation, to help code generators when generating overloaded parameter
    sets for this operation.
    """
    
    resource_type = "OperationDefinitionOverload"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.comment = None
        """ Comments to go on overload.
        Type `str`. """
        
        self.parameterName = None
        """ Name of parameter to include in overload.
        List of `str` items. """
        
        super(OperationDefinitionOverload, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(OperationDefinitionOverload, self).elementProperties()
        js.extend([
            ("comment", "comment", str, False, None, False),
            ("parameterName", "parameterName", str, True, None, False),
        ])
        return js


class OperationDefinitionParameter(backboneelement.BackboneElement):
    """ Parameters for the operation/query.
    
    The parameters for the operation/query.
    """
    
    resource_type = "OperationDefinitionParameter"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.binding = None
        """ ValueSet details if this is coded.
        Type `OperationDefinitionParameterBinding` (represented as `dict` in JSON). """
        
        self.documentation = None
        """ Description of meaning/use.
        Type `str`. """
        
        self.max = None
        """ Maximum Cardinality (a number or *).
        Type `str`. """
        
        self.min = None
        """ Minimum Cardinality.
        Type `int`. """
        
        self.name = None
        """ Name in Parameters.parameter.name or in URL.
        Type `str`. """
        
        self.part = None
        """ Parts of a nested Parameter.
        List of `OperationDefinitionParameter` items (represented as `dict` in JSON). """
        
        self.profile = None
        """ Profile on the type.
        Type `FHIRReference` referencing `StructureDefinition` (represented as `dict` in JSON). """
        
        self.searchType = None
        """ number | date | string | token | reference | composite | quantity |
        uri.
        Type `str`. """
        
        self.type = None
        """ What type this parameter has.
        Type `str`. """
        
        self.use = None
        """ in | out.
        Type `str`. """
        
        super(OperationDefinitionParameter, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(OperationDefinitionParameter, self).elementProperties()
        js.extend([
            ("binding", "binding", OperationDefinitionParameterBinding, False, None, False),
            ("documentation", "documentation", str, False, None, False),
            ("max", "max", str, False, None, True),
            ("min", "min", int, False, None, True),
            ("name", "name", str, False, None, True),
            ("part", "part", OperationDefinitionParameter, True, None, False),
            ("profile", "profile", fhirreference.FHIRReference, False, None, False),
            ("searchType", "searchType", str, False, None, False),
            ("type", "type", str, False, None, False),
            ("use", "use", str, False, None, True),
        ])
        return js


class OperationDefinitionParameterBinding(backboneelement.BackboneElement):
    """ ValueSet details if this is coded.
    
    Binds to a value set if this parameter is coded (code, Coding,
    CodeableConcept).
    """
    
    resource_type = "OperationDefinitionParameterBinding"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.strength = None
        """ required | extensible | preferred | example.
        Type `str`. """
        
        self.valueSetReference = None
        """ Source of value set.
        Type `FHIRReference` referencing `ValueSet` (represented as `dict` in JSON). """
        
        self.valueSetUri = None
        """ Source of value set.
        Type `str`. """
        
        super(OperationDefinitionParameterBinding, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(OperationDefinitionParameterBinding, self).elementProperties()
        js.extend([
            ("strength", "strength", str, False, None, True),
            ("valueSetReference", "valueSetReference", fhirreference.FHIRReference, False, "valueSet", True),
            ("valueSetUri", "valueSetUri", str, False, "valueSet", True),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import usagecontext
except ImportError:
    usagecontext = sys.modules[__package__ + '.usagecontext']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/GraphDefinition) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class GraphDefinition(domainresource.DomainResource):
    """ Definition of an graph of resources.
    
    A formal computable definition of a graph of resources - that is, a
    coherent set of resources that form a graph by following references. The
    Graph Definition resource defines a set and makes rules about the set.
    """
    
    resource_type = "GraphDefinition"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.contact = None
        """ Contact details for the publisher.
        List of `ContactDetail` items (represented as `dict` in JSON). """
        
        self.date = None
        """ Date this was last changed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.description = None
        """ Natural language description of the graph definition.
        Type `str`. """
        
        self.experimental = None
        """ For testing purposes, not real usage.
        Type `bool`. """
        
        self.jurisdiction = None
        """ Intended jurisdiction for graph definition (if applicable).
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.link = None
        """ Links this graph makes rules about.
        List of `GraphDefinitionLink` items (represented as `dict` in JSON). """
        
        self.name = None
        """ Name for this graph definition (computer friendly).
        Type `str`. """
        
        self.profile = None
        """ Profile on base resource.
        Type `str`. """
        
        self.publisher = None
        """ Name of the publisher (organization or individual).
        Type `str`. """
        
        self.purpose = None
        """ Why this graph definition is defined.
        Type `str`. """
        
        self.start = None
        """ Type of resource at which the graph starts.
        Type `str`. """
        
        self.status = None
        """ draft | active | retired | unknown.
        Type `str`. """
        
        self.url = None
        """ Logical URI to reference this graph definition (globally unique).
        Type `str`. """
        
        self.useContext = None
        """ Context the content is intended to support.
        List of `UsageContext` items (represented as `dict` in JSON). """
        
        self.version = None
        """ Business version of the graph definition.
        Type `str`. """
        
        super(GraphDefinition, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(GraphDefinition, self).elementProperties()
        js.extend([
            ("contact", "contact", contactdetail.ContactDetail, True, None, False),
            ("date", "date", fhirdate.FHIRDate, False, None, False),
            ("description", "description", str, False, None, False),
            ("experimental", "experimental", bool, False, None, False),
            ("jurisdiction", "jurisdiction", codeableconcept.CodeableConcept, True, None, False),
            ("link", "link", GraphDefinitionLink, True, None, False),
            ("name", "name", str, False, None, True),
            ("profile", "profile", str, False, None, False),
            ("publisher", "publisher", str, False, None, False),
            ("purpose", "purpose", str, False, None, False),
            ("start", "start", str, False, None, True),
            ("status", "status", str, False, None, True),
            ("url", "url", str, False, None, False),
            ("useContext", "useContext", usagecontext.UsageContext, True, None, False),
            ("version", "version", str, False, None, False),
        ])
        return js


from . import backboneelement

class GraphDefinitionLink(backboneelement.BackboneElement):
    """ Links this graph makes rules about.
    """
    
    resource_type = "GraphDefinitionLink"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.description = None
        """ Why this link is specified.
        Type `str`. """
        
        self.max = None
        """ Maximum occurrences for this link.
        Type `str`. """
        
        self.min = None
        """ Minimum occurrences for this link.
        Type `int`. """
        
        self.path = None
        """ Path in the resource that contains the link.
        Type `str`. """
        
        self.sliceName = None
        """ Which slice (if profiled).
        Type `str`. """
        
        self.target = None
        """ Potential target for the link.
        List of `GraphDefinitionLinkTarget` items (represented as `dict` in JSON). """
        
        super(GraphDefinitionLink, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(GraphDefinitionLink, self).elementProperties()
        js.extend([
            ("description", "description", str, False, None, False),
            ("max", "max", str, False, None, False),
            ("min", "min", int, False, None, False),
            ("path", "path", str, False, None, True),
            ("sliceName", "sliceName", str, False, None, False),
            ("target", "target", GraphDefinitionLinkTarget, True, None, True),
        ])
        return js


class GraphDefinitionLinkTarget(backboneelement.BackboneElement):
    """ Potential target for the link.
    """
    
    resource_type = "GraphDefinitionLinkTarget"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.compartment = None
        """ Compartment Consistency Rules.
        List of `GraphDefinitionLinkTargetCompartment` items (represented as `dict` in JSON). """
        
        self.link = None
        """ Additional links from target resource.
        List of `GraphDefinitionLink` items (represented as `dict` in JSON). """
        
        self.profile = None
        """ Profile for the target resource.
        Type `str`. """
        
        self.type = None
        """ Type of resource this link refers to.
        Type `str`. """
        
        super(GraphDefinitionLinkTarget, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(GraphDefinitionLinkTarget, self).elementProperties()
        js.extend([
            ("compartment", "compartment", GraphDefinitionLinkTargetCompartment, True, None, False),
            ("link", "link", GraphDefinitionLink, True, None, False),
            ("profile", "profile", str, False, None, False),
            ("type", "type", str, False, None, True),
        ])
        return js


class GraphDefinitionLinkTargetCompartment(backboneelement.BackboneElement):
    """ Compartment Consistency Rules.
    """
    
    resource_type = "GraphDefinitionLinkTargetCompartment"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.code = None
        """ Identifies the compartment.
        Type `str`. """
        
        self.description = None
        """ Documentation for FHIRPath expression.
        Type `str`. """
        
        self.expression = None
        """ Custom rule, as a FHIRPath expression.
        Type `str`. """
        
        self.rule = None
        """ identical | matching | different | custom.
        Type `str`. """
        
        super(GraphDefinitionLinkTargetCompartment, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(GraphDefinitionLinkTargetCompartment, self).elementProperties()
        js.extend([
            ("code", "code", str, False, None, True),
            ("description", "description", str, False, None, False),
            ("expression", "expression", str, False, None, False),
            ("rule", "rule", str, False, None, True),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import usagecontext
except ImportError:
    usagecontext = sys.modules[__package__ + '.usagecontext']



import hashlib
import io
import os
import shutil
import sys
import tempfile
import zipfile

import pytest
import six

ROOT_PATH = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
PARSER_PATH = os.path.join(ROOT_PATH, 'fhir-parser')
SCRIPT_PATH = os.path.join(ROOT_PATH, 'script')
CACHE_PATH = os.path.join(ROOT_PATH, '.cache')


def force_bytes(string, encoding='utf8', errors='strict'):

    if isinstance(string, bytes):
        if encoding == 'utf8':
            return string
        else:
            return string.decode('utf8', errors).encode(encoding, errors)

    if not isinstance(string, six.string_types):
        return string

    return string.encode(encoding, errors)


def download_and_store(url, path):
    """ """
    import requests
    try:
        sys.stdout.write('Attempting to download from {0}\n'.format(url))
        ret = requests.get(url)
    except requests.HTTPError as exc:
        raise LookupError(
            'Failed to download. Full error: {0!s}'.format(exc)
        )
    else:
        if not ret.ok:
            raise Exception("Failed to download {0}".format(url))
        with io.open(path, 'wb') as handle:
            for chunk in ret.iter_content():
                handle.write(chunk)
            sys.stdout.write(
                'Download has been completed, now saved to {0}\n'.format(path)
            )


def expand(self, local):
    """ Expand the ZIP file at the given path to the cache directory.
    """
    path = os.path.join(self.cache, local)
    assert os.path.exists(path)
    import zipfile  # import here as we can bypass its use with a manual unzip

    with zipfile.ZipFile(path) as z:
        z.extractall(self.cache)


@pytest.fixture(scope='session')
def base_settings():

    if not os.path.exists(CACHE_PATH):
        os.mkdir(CACHE_PATH)

    settings = {}

    with io.open(os.path.join(SCRIPT_PATH, 'settings.py'), 'r') as fp:
        exec(fp.read(), settings)

    example_data_file_uri = settings['specification_url'] + '/examples-json.zip'

    example_data_file_id = \
            hashlib.md5(example_data_file_uri.encode()).hexdigest()

    example_data_file_location = os.path.join(CACHE_PATH, (example_data_file_id + '.zip'))

    if not os.path.exists(example_data_file_location):
        download_and_store(
            example_data_file_uri,
            example_data_file_location)

    temp_data_dir = tempfile.mkdtemp()
    # extract all files from archive and put into temp dir
    with zipfile.ZipFile(example_data_file_location) as z:
        z.extractall(temp_data_dir)

    if 'FHIR_UNITTEST_DATADIR' not in os.environ:
        os.environ.setdefault('FHIR_UNITTEST_DATADIR', temp_data_dir)

    yield settings

    shutil.rmtree(temp_data_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Library) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class Library(domainresource.DomainResource):
    """ Represents a library of quality improvement components.
    
    The Library resource is a general-purpose container for knowledge asset
    definitions. It can be used to describe and expose existing knowledge
    assets such as logic libraries and information model descriptions, as well
    as to describe a collection of knowledge assets.
    """
    
    resource_type = "Library"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.approvalDate = None
        """ When the library was approved by publisher.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.contact = None
        """ Contact details for the publisher.
        List of `ContactDetail` items (represented as `dict` in JSON). """
        
        self.content = None
        """ Contents of the library, either embedded or referenced.
        List of `Attachment` items (represented as `dict` in JSON). """
        
        self.contributor = None
        """ A content contributor.
        List of `Contributor` items (represented as `dict` in JSON). """
        
        self.copyright = None
        """ Use and/or publishing restrictions.
        Type `str`. """
        
        self.dataRequirement = None
        """ What data is referenced by this library.
        List of `DataRequirement` items (represented as `dict` in JSON). """
        
        self.date = None
        """ Date this was last changed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.description = None
        """ Natural language description of the library.
        Type `str`. """
        
        self.effectivePeriod = None
        """ When the library is expected to be used.
        Type `Period` (represented as `dict` in JSON). """
        
        self.experimental = None
        """ For testing purposes, not real usage.
        Type `bool`. """
        
        self.identifier = None
        """ Additional identifier for the library.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.jurisdiction = None
        """ Intended jurisdiction for library (if applicable).
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.lastReviewDate = None
        """ When the library was last reviewed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.name = None
        """ Name for this library (computer friendly).
        Type `str`. """
        
        self.parameter = None
        """ Parameters defined by the library.
        List of `ParameterDefinition` items (represented as `dict` in JSON). """
        
        self.publisher = None
        """ Name of the publisher (organization or individual).
        Type `str`. """
        
        self.purpose = None
        """ Why this library is defined.
        Type `str`. """
        
        self.relatedArtifact = None
        """ Additional documentation, citations, etc..
        List of `RelatedArtifact` items (represented as `dict` in JSON). """
        
        self.status = None
        """ draft | active | retired | unknown.
        Type `str`. """
        
        self.title = None
        """ Name for this library (human friendly).
        Type `str`. """
        
        self.topic = None
        """ E.g. Education, Treatment, Assessment, etc.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.type = None
        """ logic-library | model-definition | asset-collection | module-
        definition.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.url = None
        """ Logical URI to reference this library (globally unique).
        Type `str`. """
        
        self.usage = None
        """ Describes the clinical usage of the library.
        Type `str`. """
        
        self.useContext = None
        """ Context the content is intended to support.
        List of `UsageContext` items (represented as `dict` in JSON). """
        
        self.version = None
        """ Business version of the library.
        Type `str`. """
        
        super(Library, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Library, self).elementProperties()
        js.extend([
            ("approvalDate", "approvalDate", fhirdate.FHIRDate, False, None, False),
            ("contact", "contact", contactdetail.ContactDetail, True, None, False),
            ("content", "content", attachment.Attachment, True, None, False),
            ("contributor", "contributor", contributor.Contributor, True, None, False),
            ("copyright", "copyright", str, False, None, False),
            ("dataRequirement", "dataRequirement", datarequirement.DataRequirement, True, None, False),
            ("date", "date", fhirdate.FHIRDate, False, None, False),
            ("description", "description", str, False, None, False),
            ("effectivePeriod", "effectivePeriod", period.Period, False, None, False),
            ("experimental", "experimental", bool, False, None, False),
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("jurisdiction", "jurisdiction", codeableconcept.CodeableConcept, True, None, False),
            ("lastReviewDate", "lastReviewDate", fhirdate.FHIRDate, False, None, False),
            ("name", "name", str, False, None, False),
            ("parameter", "parameter", parameterdefinition.ParameterDefinition, True, None, False),
            ("publisher", "publisher", str, False, None, False),
            ("purpose", "purpose", str, False, None, False),
            ("relatedArtifact", "relatedArtifact", relatedartifact.RelatedArtifact, True, None, False),
            ("status", "status", str, False, None, True),
            ("title", "title", str, False, None, False),
            ("topic", "topic", codeableconcept.CodeableConcept, True, None, False),
            ("type", "type", codeableconcept.CodeableConcept, False, None, True),
            ("url", "url", str, False, None, False),
            ("usage", "usage", str, False, None, False),
            ("useContext", "useContext", usagecontext.UsageContext, True, None, False),
            ("version", "version", str, False, None, False),
        ])
        return js


import sys
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']
try:
    from . import contributor
except ImportError:
    contributor = sys.modules[__package__ + '.contributor']
try:
    from . import datarequirement
except ImportError:
    datarequirement = sys.modules[__package__ + '.datarequirement']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import parameterdefinition
except ImportError:
    parameterdefinition = sys.modules[__package__ + '.parameterdefinition']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import relatedartifact
except ImportError:
    relatedartifact = sys.modules[__package__ + '.relatedartifact']
try:
    from . import usagecontext
except ImportError:
    usagecontext = sys.modules[__package__ + '.usagecontext']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import supplydelivery
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class SupplyDeliveryTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("SupplyDelivery", js["resourceType"])
        return supplydelivery.SupplyDelivery(js)
    
    def testSupplyDelivery1(self):
        inst = self.instantiate_from("supplydelivery-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a SupplyDelivery instance")
        self.implSupplyDelivery1(inst)
        
        js = inst.as_json()
        self.assertEqual("SupplyDelivery", js["resourceType"])
        inst2 = supplydelivery.SupplyDelivery(js)
        self.implSupplyDelivery1(inst2)
    
    def implSupplyDelivery1(self, inst):
        self.assertEqual(force_bytes(inst.id), force_bytes("simpledelivery"))
        self.assertEqual(force_bytes(inst.identifier.value), force_bytes("Order10284"))
        self.assertEqual(inst.occurrenceDateTime.date, FHIRDate("2016-12-31").date)
        self.assertEqual(inst.occurrenceDateTime.as_json(), "2016-12-31")
        self.assertEqual(force_bytes(inst.status), force_bytes("completed"))
        self.assertEqual(force_bytes(inst.suppliedItem.itemCodeableConcept.coding[0].code), force_bytes("BlueTubes"))
        self.assertEqual(force_bytes(inst.suppliedItem.itemCodeableConcept.coding[0].display), force_bytes("Blood collect tubes blue cap"))
        self.assertEqual(inst.suppliedItem.quantity.value, 10)
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("device"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://hl7.org/fhir/supply-item-type"))
        self.assertEqual(force_bytes(inst.type.text), force_bytes("Blood collect tubes blue cap"))
    
    def testSupplyDelivery2(self):
        inst = self.instantiate_from("supplydelivery-example-pumpdelivery.json")
        self.assertIsNotNone(inst, "Must have instantiated a SupplyDelivery instance")
        self.implSupplyDelivery2(inst)
        
        js = inst.as_json()
        self.assertEqual("SupplyDelivery", js["resourceType"])
        inst2 = supplydelivery.SupplyDelivery(js)
        self.implSupplyDelivery2(inst2)
    
    def implSupplyDelivery2(self, inst):
        self.assertEqual(force_bytes(inst.id), force_bytes("pumpdelivery"))
        self.assertEqual(force_bytes(inst.identifier.value), force_bytes("98398459409"))
        self.assertEqual(force_bytes(inst.status), force_bytes("in-progress"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">[Put rendering here]</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/MedicationRequest) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class MedicationRequest(domainresource.DomainResource):
    """ Ordering of medication for patient or group.
    
    An order or request for both supply of the medication and the instructions
    for administration of the medication to a patient. The resource is called
    "MedicationRequest" rather than "MedicationPrescription" or
    "MedicationOrder" to generalize the use across inpatient and outpatient
    settings, including care plans, etc., and to harmonize with workflow
    patterns.
    """
    
    resource_type = "MedicationRequest"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.authoredOn = None
        """ When request was initially authored.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.basedOn = None
        """ What request fulfills.
        List of `FHIRReference` items referencing `CarePlan, MedicationRequest, ProcedureRequest, ReferralRequest` (represented as `dict` in JSON). """
        
        self.category = None
        """ Type of medication usage.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.context = None
        """ Created during encounter/admission/stay.
        Type `FHIRReference` referencing `Encounter, EpisodeOfCare` (represented as `dict` in JSON). """
        
        self.definition = None
        """ Protocol or definition.
        List of `FHIRReference` items referencing `ActivityDefinition, PlanDefinition` (represented as `dict` in JSON). """
        
        self.detectedIssue = None
        """ Clinical Issue with action.
        List of `FHIRReference` items referencing `DetectedIssue` (represented as `dict` in JSON). """
        
        self.dispenseRequest = None
        """ Medication supply authorization.
        Type `MedicationRequestDispenseRequest` (represented as `dict` in JSON). """
        
        self.dosageInstruction = None
        """ How the medication should be taken.
        List of `Dosage` items (represented as `dict` in JSON). """
        
        self.eventHistory = None
        """ A list of events of interest in the lifecycle.
        List of `FHIRReference` items referencing `Provenance` (represented as `dict` in JSON). """
        
        self.groupIdentifier = None
        """ Composite request this is part of.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.identifier = None
        """ External ids for this request.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.intent = None
        """ proposal | plan | order | instance-order.
        Type `str`. """
        
        self.medicationCodeableConcept = None
        """ Medication to be taken.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.medicationReference = None
        """ Medication to be taken.
        Type `FHIRReference` referencing `Medication` (represented as `dict` in JSON). """
        
        self.note = None
        """ Information about the prescription.
        List of `Annotation` items (represented as `dict` in JSON). """
        
        self.priorPrescription = None
        """ An order/prescription that is being replaced.
        Type `FHIRReference` referencing `MedicationRequest` (represented as `dict` in JSON). """
        
        self.priority = None
        """ routine | urgent | stat | asap.
        Type `str`. """
        
        self.reasonCode = None
        """ Reason or indication for writing the prescription.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.reasonReference = None
        """ Condition or Observation that supports why the prescription is
        being written.
        List of `FHIRReference` items referencing `Condition, Observation` (represented as `dict` in JSON). """
        
        self.recorder = None
        """ Person who entered the request.
        Type `FHIRReference` referencing `Practitioner` (represented as `dict` in JSON). """
        
        self.requester = None
        """ Who/What requested the Request.
        Type `MedicationRequestRequester` (represented as `dict` in JSON). """
        
        self.status = None
        """ active | on-hold | cancelled | completed | entered-in-error |
        stopped | draft | unknown.
        Type `str`. """
        
        self.subject = None
        """ Who or group medication request is for.
        Type `FHIRReference` referencing `Patient, Group` (represented as `dict` in JSON). """
        
        self.substitution = None
        """ Any restrictions on medication substitution.
        Type `MedicationRequestSubstitution` (represented as `dict` in JSON). """
        
        self.supportingInformation = None
        """ Information to support ordering of the medication.
        List of `FHIRReference` items referencing `Resource` (represented as `dict` in JSON). """
        
        super(MedicationRequest, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationRequest, self).elementProperties()
        js.extend([
            ("authoredOn", "authoredOn", fhirdate.FHIRDate, False, None, False),
            ("basedOn", "basedOn", fhirreference.FHIRReference, True, None, False),
            ("category", "category", codeableconcept.CodeableConcept, False, None, False),
            ("context", "context", fhirreference.FHIRReference, False, None, False),
            ("definition", "definition", fhirreference.FHIRReference, True, None, False),
            ("detectedIssue", "detectedIssue", fhirreference.FHIRReference, True, None, False),
            ("dispenseRequest", "dispenseRequest", MedicationRequestDispenseRequest, False, None, False),
            ("dosageInstruction", "dosageInstruction", dosage.Dosage, True, None, False),
            ("eventHistory", "eventHistory", fhirreference.FHIRReference, True, None, False),
            ("groupIdentifier", "groupIdentifier", identifier.Identifier, False, None, False),
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("intent", "intent", str, False, None, True),
            ("medicationCodeableConcept", "medicationCodeableConcept", codeableconcept.CodeableConcept, False, "medication", True),
            ("medicationReference", "medicationReference", fhirreference.FHIRReference, False, "medication", True),
            ("note", "note", annotation.Annotation, True, None, False),
            ("priorPrescription", "priorPrescription", fhirreference.FHIRReference, False, None, False),
            ("priority", "priority", str, False, None, False),
            ("reasonCode", "reasonCode", codeableconcept.CodeableConcept, True, None, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, True, None, False),
            ("recorder", "recorder", fhirreference.FHIRReference, False, None, False),
            ("requester", "requester", MedicationRequestRequester, False, None, False),
            ("status", "status", str, False, None, False),
            ("subject", "subject", fhirreference.FHIRReference, False, None, True),
            ("substitution", "substitution", MedicationRequestSubstitution, False, None, False),
            ("supportingInformation", "supportingInformation", fhirreference.FHIRReference, True, None, False),
        ])
        return js


from . import backboneelement

class MedicationRequestDispenseRequest(backboneelement.BackboneElement):
    """ Medication supply authorization.
    
    Indicates the specific details for the dispense or medication supply part
    of a medication request (also known as a Medication Prescription or
    Medication Order).  Note that this information is not always sent with the
    order.  There may be in some settings (e.g. hospitals) institutional or
    system support for completing the dispense details in the pharmacy
    department.
    """
    
    resource_type = "MedicationRequestDispenseRequest"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.expectedSupplyDuration = None
        """ Number of days supply per dispense.
        Type `Duration` (represented as `dict` in JSON). """
        
        self.numberOfRepeatsAllowed = None
        """ Number of refills authorized.
        Type `int`. """
        
        self.performer = None
        """ Intended dispenser.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """
        
        self.quantity = None
        """ Amount of medication to supply per dispense.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.validityPeriod = None
        """ Time period supply is authorized for.
        Type `Period` (represented as `dict` in JSON). """
        
        super(MedicationRequestDispenseRequest, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationRequestDispenseRequest, self).elementProperties()
        js.extend([
            ("expectedSupplyDuration", "expectedSupplyDuration", duration.Duration, False, None, False),
            ("numberOfRepeatsAllowed", "numberOfRepeatsAllowed", int, False, None, False),
            ("performer", "performer", fhirreference.FHIRReference, False, None, False),
            ("quantity", "quantity", quantity.Quantity, False, None, False),
            ("validityPeriod", "validityPeriod", period.Period, False, None, False),
        ])
        return js


class MedicationRequestRequester(backboneelement.BackboneElement):
    """ Who/What requested the Request.
    
    The individual, organization or device that initiated the request and has
    responsibility for its activation.
    """
    
    resource_type = "MedicationRequestRequester"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.agent = None
        """ Who ordered the initial medication(s).
        Type `FHIRReference` referencing `Practitioner, Organization, Patient, RelatedPerson, Device` (represented as `dict` in JSON). """
        
        self.onBehalfOf = None
        """ Organization agent is acting for.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """
        
        super(MedicationRequestRequester, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationRequestRequester, self).elementProperties()
        js.extend([
            ("agent", "agent", fhirreference.FHIRReference, False, None, True),
            ("onBehalfOf", "onBehalfOf", fhirreference.FHIRReference, False, None, False),
        ])
        return js


class MedicationRequestSubstitution(backboneelement.BackboneElement):
    """ Any restrictions on medication substitution.
    
    Indicates whether or not substitution can or should be part of the
    dispense. In some cases substitution must happen, in other cases
    substitution must not happen. This block explains the prescriber's intent.
    If nothing is specified substitution may be done.
    """
    
    resource_type = "MedicationRequestSubstitution"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.allowed = None
        """ Whether substitution is allowed or not.
        Type `bool`. """
        
        self.reason = None
        """ Why should (not) substitution be made.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(MedicationRequestSubstitution, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(MedicationRequestSubstitution, self).elementProperties()
        js.extend([
            ("allowed", "allowed", bool, False, None, True),
            ("reason", "reason", codeableconcept.CodeableConcept, False, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import dosage
except ImportError:
    dosage = sys.modules[__package__ + '.dosage']
try:
    from . import duration
except ImportError:
    duration = sys.modules[__package__ + '.duration']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Reference) on 2019-01-12.
#  2019, SMART Health IT.


from . import element

class Reference(element.Element):
    """ A reference from one resource to another.
    """
    
    resource_type = "Reference"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.display = None
        """ Text alternative for the resource.
        Type `str`. """
        
        self.identifier = None
        """ Logical reference, when literal reference is not known.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.reference = None
        """ Literal reference, Relative, internal or absolute URL.
        Type `str`. """
        
        super(Reference, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Reference, self).elementProperties()
        js.extend([
            ("display", "display", str, False, None, False),
            ("identifier", "identifier", identifier.Identifier, False, None, False),
            ("reference", "reference", str, False, None, False),
        ])
        return js


import sys
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
